@@ -1,13 +1,7 @@
 /***************************************************************************
-                          directorymergewindow.h
-                             -------------------
-    begin                : Sat Oct 19 2002
-    copyright            : (C) 2002-2007 by Joachim Eibl
-    email                : joachim.eibl at gmx.de
- ***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
+ *   Copyright (C) 2003-2007 by Joachim Eibl <joachim.eibl at gmx.de>      *
+ *   Copyright (C) 2018 Michael Reeves reeves.87@gmail.com                 *
+ *                                                                         *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
@@ -32,41 +26,24 @@ class StatusInfo;
 class DirectoryMergeInfo;
 class OneDirectoryInfo;
 class QLabel;
+class QAction;
 class KToggleAction;
 class KActionCollection;
 class TotalDiffStatus;
-
-enum e_MergeOperation
-{
-   eTitleId,
-   eNoOperation,
-   // Operations in sync mode (with only two directories):
-   eCopyAToB, eCopyBToA, eDeleteA, eDeleteB, eDeleteAB, eMergeToA, eMergeToB, eMergeToAB,
-
-   // Operations in merge mode (with two or three directories)
-   eCopyAToDest, eCopyBToDest, eCopyCToDest, eDeleteFromDest, eMergeABCToDest,
-   eMergeABToDest,
-   eConflictingFileTypes, // Error
-   eChangedAndDeleted,    // Error
-   eConflictingAges       // Equal age but files are not!
-};
-
-enum e_Age { eNew, eMiddle, eOld, eNotThere, eAgeEnd };
+class DirectoryInfo;
 
 class MergeFileInfos;
 
+class KDiff3App;
 class DirectoryMergeWindow : public QTreeView
 {
    Q_OBJECT
 public:
-   DirectoryMergeWindow( QWidget* pParent, Options* pOptions, KIconLoader* pIconLoader );
-   ~DirectoryMergeWindow();
+   DirectoryMergeWindow( QWidget* pParent, Options* pOptions );
+   ~DirectoryMergeWindow() override;
    void setDirectoryMergeInfo(DirectoryMergeInfo* p);
    bool init(
-      FileAccess& dirA,
-      FileAccess& dirB,
-      FileAccess& dirC,
-      FileAccess& dirDest,
+      const QSharedPointer<DirectoryInfo> &dirInfo,
       bool bDirectoryMerge,
       bool bReload = false
    );
@@ -75,22 +52,23 @@ public:
    int totalColumnWidth();
    bool isSyncMode();
    bool isScanning();
-   void initDirectoryMergeActions( QObject* pKDiff3App, KActionCollection* ac );
+   void initDirectoryMergeActions( KDiff3App* pKDiff3App, KActionCollection* ac );
    void updateAvailabilities( bool bDirCompare, bool bDiffWindowVisible,
       KToggleAction* chooseA, KToggleAction* chooseB, KToggleAction* chooseC );
    void updateFileVisibilities();
 
-   virtual void mousePressEvent( QMouseEvent* e );
-   virtual void keyPressEvent( QKeyEvent* e );
-   virtual void focusInEvent( QFocusEvent* e );
-   virtual void focusOutEvent( QFocusEvent* e );
-   virtual void contextMenuEvent( QContextMenuEvent* e );
-   QString getDirNameA();
-   QString getDirNameB();
-   QString getDirNameC();
-   QString getDirNameDest();
+   void mousePressEvent( QMouseEvent* e ) override;
+   void keyPressEvent( QKeyEvent* e ) override;
+   void focusInEvent( QFocusEvent* e ) override;
+   void focusOutEvent( QFocusEvent* e ) override;
+   void contextMenuEvent( QContextMenuEvent* e ) override;
 
-public slots:
+   QString getDirNameA() const;
+   QString getDirNameB() const;
+   QString getDirNameC() const;
+   QString getDirNameDest() const;
+
+ public Q_SLOTS:
    void reload();
    void mergeCurrentFile();
    void compareCurrentFile();
@@ -136,39 +114,41 @@ public slots:
    void slotSaveMergeState();
    void slotLoadMergeState();
 
-signals:
+Q_SIGNALS:
    void startDiffMerge(QString fn1,QString fn2, QString fn3, QString ofn, QString,QString,QString,TotalDiffStatus*);
    void checkIfCanContinue( bool* pbContinue );
    void updateAvailabilities();
    void statusBarMessage( const QString& msg );
-protected slots:
+protected Q_SLOTS:
    void onDoubleClick( const QModelIndex& );
    void onExpanded();
-   void	currentChanged( const QModelIndex & current, const QModelIndex & previous ); // override
+   void	currentChanged( const QModelIndex & current, const QModelIndex & previous ) override; // override
 private:
-   class Data;
-   friend class Data;
-   Data* d;
-   class DirMergeItemDelegate;
-   friend class DirMergeItemDelegate;
+  class DirectoryMergeWindowPrivate;
+  friend class DirectoryMergeWindowPrivate;
+  DirectoryMergeWindowPrivate* d;
+  class DirMergeItemDelegate;
+  friend class DirMergeItemDelegate;
 };
 
 class DirectoryMergeInfo : public QFrame
 {
    Q_OBJECT
 public:
-   DirectoryMergeInfo( QWidget* pParent );
+   explicit DirectoryMergeInfo( QWidget* pParent );
    void setInfo(
-      const FileAccess& APath,
-      const FileAccess& BPath,
-      const FileAccess& CPath,
-      const FileAccess& DestPath,
+      const FileAccess& dirA,
+      const FileAccess& dirB,
+      const FileAccess& dirC,
+      const FileAccess& dirDest,
       MergeFileInfos& mfi );
    QTreeWidget* getInfoList() {return m_pInfoList;}
-   virtual bool eventFilter( QObject* o, QEvent* e );
-signals:
+   bool eventFilter( QObject* o, QEvent* e ) override;
+Q_SIGNALS:
    void gotFocus();
 private:
+   void addListViewItem(const QString& dir, const QString& basePath, FileAccess* fi);
+
    QLabel* m_pInfoA;
    QLabel* m_pInfoB;
    QLabel* m_pInfoC;
