@@ -10,6 +10,7 @@
 #include "fileaccess.h"
 #include "cvsignorelist.h"
 #include "common.h"
+#include "Logging.h"
 #include "progress.h"
 #include "Utils.h"
 
@@ -51,7 +52,9 @@ void FileAccess::reset()
     m_bFile = false;
     m_bDir = false;
     m_bSymLink = false;
+    m_bReadable = false;
     m_bWritable = false;
+    m_bExecutable = false;
     m_bHidden = false;
     m_size = 0;
     m_modificationTime = QDateTime();
@@ -62,7 +65,6 @@ void FileAccess::reset()
 
     m_linkTarget = "";
     //m_fileType = -1;
-    m_pParent = nullptr;
     tmpFile.clear();
     tmpFile = QSharedPointer<QTemporaryFile>::create();
     realFile = nullptr;
@@ -101,6 +103,7 @@ void FileAccess::setFile(const QString& name, bool bWantToWrite)
 void FileAccess::setFile(const QUrl& url, bool bWantToWrite)
 {
     reset();
+    Q_ASSERT(parent() == nullptr || url != parent()->url());
 
     m_url = url;
     //QUrl::isLocalFile assumes the scheme is set.
@@ -131,7 +134,7 @@ void FileAccess::loadData()
     m_fileInfo.setCaching(true);
 
     if(parent() == nullptr)
-        m_baseDir = m_fileInfo.absoluteFilePath();
+        m_baseDir.setPath(m_fileInfo.absoluteFilePath());
     else
         m_baseDir = m_pParent->m_baseDir;
 
@@ -158,7 +161,7 @@ void FileAccess::loadData()
 
     if(isLocal() && m_bSymLink)
     {
-        m_linkTarget = m_fileInfo.readLink();
+        m_linkTarget = m_fileInfo.symLinkTarget();
 #ifndef Q_OS_WIN
         // Unfortunately Qt5 symLinkTarget/readLink always returns an absolute path, even if the link is relative
         char* s = (char*)malloc(PATH_MAX + 1);
@@ -281,7 +284,24 @@ void FileAccess::setFromUdsEntry(const KIO::UDSEntry& e, FileAccess *parent)
     m_fileInfo = QFileInfo(filePath);
     m_fileInfo.setCaching(true);
     if(m_url.isEmpty())
-        m_url = QUrl::fromUserInput(filePath);
+    {
+        if(parent != nullptr)
+        {
+            m_url = parent->url().resolved(QUrl(filePath));
+            //Verify that the scheme doesn't change.
+            Q_ASSERT(m_url.scheme() == parent->url().scheme());
+        }
+        else
+        {
+            /*
+             Invalid entry we don't know the full url because KIO didn't tell us and there is no parent
+             node supplied.
+             This is a bug if it happens and should be logged . However it is a recoverable error.
+            */
+            qCWarning(kdiffFileAccess) << i18n("Unable to determine full url. No parent specified.");
+            return;
+        }
+    }
 
     m_name = m_fileInfo.fileName();
     if(isLocal() && m_name.isEmpty())
@@ -473,6 +493,7 @@ bool FileAccess::interruptableReadFile(void* pDestBuffer, qint64 maxLength)
 
 bool FileAccess::readFile(void* pDestBuffer, qint64 maxLength)
 {
+    bool success = false;
     //Avoid hang on linux for special files.
     if(!isNormal())
         return true;
@@ -480,14 +501,20 @@ bool FileAccess::readFile(void* pDestBuffer, qint64 maxLength)
     if(isLocal() || !m_localCopy.isEmpty())
     {
         if(open(QIODevice::ReadOnly))//krazy:exclude=syscalls
-            return interruptableReadFile(pDestBuffer, maxLength); // maxLength == f.read( (char*)pDestBuffer, maxLength );
+        {
+            success = interruptableReadFile(pDestBuffer, maxLength); // maxLength == f.read( (char*)pDestBuffer, maxLength )
+            close();
+        }
     }
     else
     {
         FileAccessJobHandler jh(this);
-        return jh.get(pDestBuffer, maxLength);
+        success = jh.get(pDestBuffer, maxLength);
     }
-    return false;
+
+    close();
+    Q_ASSERT(!realFile->isOpen() && !tmpFile->isOpen());
+    return success;
 }
 
 bool FileAccess::writeFile(const void* pSrcBuffer, qint64 length)
@@ -495,8 +522,7 @@ bool FileAccess::writeFile(const void* pSrcBuffer, qint64 length)
     ProgressProxy pp;
     if(isLocal())
     {
-        QFile f(absoluteFilePath());
-        if(f.open(QIODevice::WriteOnly))
+        if(realFile->open(QIODevice::WriteOnly))
         {
             const qint64 maxChunkSize = 100000;
             pp.setMaxNofSteps(length / maxChunkSize + 1);
@@ -504,33 +530,44 @@ bool FileAccess::writeFile(const void* pSrcBuffer, qint64 length)
             while(i < length)
             {
                 qint64 nextLength = std::min(length - i, maxChunkSize);
-                qint64 reallyWritten = f.write((char*)pSrcBuffer + i, nextLength);
+                qint64 reallyWritten = realFile->write((char*)pSrcBuffer + i, nextLength);
                 if(reallyWritten != nextLength)
                 {
+                    realFile->close();
                     return false;
                 }
                 i += reallyWritten;
 
                 pp.step();
                 if(pp.wasCancelled())
+                {
+                    realFile->close();
                     return false;
+                }
             }
-            f.close();
 
             if(isExecutable()) // value is true if the old file was executable
             {
                 // Preserve attributes
-                f.setPermissions(f.permissions() | QFile::ExeUser);
+                realFile->setPermissions(realFile->permissions() | QFile::ExeUser);
             }
 
+            realFile->close();
             return true;
         }
     }
     else
     {
         FileAccessJobHandler jh(this);
-        return jh.put(pSrcBuffer, length, true /*overwrite*/);
+        bool success = jh.put(pSrcBuffer, length, true /*overwrite*/);
+        close();
+
+        Q_ASSERT(!realFile->isOpen() && !tmpFile->isOpen());
+
+        return success;
     }
+    close();
+    Q_ASSERT(!realFile->isOpen() && !tmpFile->isOpen());
     return false;
 }
 
@@ -815,7 +852,7 @@ FileAccessJobHandler::FileAccessJobHandler(FileAccess* pFileAccess)
     m_bSuccess = false;
 }
 
-bool FileAccessJobHandler::stat(int detail, bool bWantToWrite)
+bool FileAccessJobHandler::stat(short detail, bool bWantToWrite)
 {
     m_bSuccess = false;
     m_pFileAccess->setStatusText(QString());
@@ -1121,8 +1158,11 @@ bool FileAccessJobHandler::listDir(t_DirectoryList* pDirList, bool bRecursive, b
         QFileInfoList fiList = dir.entryInfoList();
         if(fiList.isEmpty())
         {
-            // No Permission to read directory or other error.
-            m_bSuccess = false;
+            /*
+                Sadly Qt provides no error information making this case ambigious.
+                A readablity check is the best we can do.
+            */
+            m_bSuccess = dir.isReadable();
         }
         else
         {
@@ -1203,7 +1243,8 @@ void FileAccessJobHandler::slotListDirProcessNewEntries(KIO::Job*, const KIO::UD
         //must be manually filtered KDE does not supply API for ignoring these.
         if(fa.fileName() != "." && fa.fileName() != "..")
         {
-            fa.addPath(fa.fileName());
+            //quick fix to preserve behavoir without creating invalid urls. TODO: look for altertive machanism for use with next major release.
+            fa.setFile(fa.url());
 
             m_pDirList->push_back(fa);
         }
