@@ -117,10 +117,10 @@ class FileAccess
     bool interruptableReadFile(void* pDestBuffer, qint64 maxLength);
 
     QUrl m_url;
-    bool m_bValidData;
+    bool m_bValidData = false;
 
     //long m_fileType; // for testing only
-    FileAccess* m_pParent;
+    FileAccess* m_pParent = nullptr;
 
     QDir m_baseDir;
     QFileInfo m_fileInfo;
@@ -130,16 +130,16 @@ class FileAccess
     QSharedPointer<QTemporaryFile> tmpFile = nullptr;
     QSharedPointer<QFile> realFile = nullptr;
 
-    qint64 m_size;
+    qint64 m_size = 0;
     QDateTime m_modificationTime;
-    bool m_bSymLink;
-    bool m_bFile;
-    bool m_bDir;
-    bool m_bExists;
-    bool m_bWritable;
-    bool m_bReadable;
-    bool m_bExecutable;
-    bool m_bHidden;
+    bool m_bSymLink = false;
+    bool m_bFile = false;
+    bool m_bDir = false;
+    bool m_bExists = false;
+    bool m_bWritable = false;
+    bool m_bReadable = false;
+    bool m_bExecutable = false;
+    bool m_bHidden = false;
 
     QString m_statusText; // Might contain an error string, when the last operation didn't succeed.
 };
@@ -156,7 +156,7 @@ class FileAccessJobHandler : public QObject
 
     bool get(void* pDestBuffer, long maxLength);
     bool put(const void* pSrcBuffer, long maxLength, bool bOverwrite, bool bResume = false, int permissions = -1);
-    bool stat(int detailLevel = 2, bool bWantToWrite = false);
+    bool stat(short detailLevel = 2, bool bWantToWrite = false);
     bool copyFile(const QString& dest);
     bool rename(const FileAccess& dest);
     bool listDir(t_DirectoryList* pDirList, bool bRecursive, bool bFindHidden,
